@@ -12,7 +12,7 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
-pub mod uploader;
+pub mod remote;
 
 pub mod aws;
 pub mod ssh;
